#========================================================================================
# package      DE_EPAGES::Amazon::API::Object::RemoteMerchantToMWSAdapter
# state        public
#----------------------------------------------------------------------------------------
# description  Amazon MWS API Adapter. Compatible interface to RemoteMerchant
#========================================================================================
package DE_EPAGES::Amazon::API::Object::RemoteMerchantToMWSAdapter;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::TestParameter qw ( TestString );
use DE_EPAGES::Amazon::API::MWS::GetOrderServiceStatusResponseHandler;
use DE_EPAGES::Amazon::API::MWS::ManageReportScheduleResponseHandler;
use DE_EPAGES::Amazon::API::MWS::SubmitFeedResponseHandler;
use DE_EPAGES::Amazon::API::MWS::GetFeedSubmissionResultResponseHandler;
use DE_EPAGES::Amazon::API::MWS::GetFeedSubmissionListResponseHandler;
use DE_EPAGES::Amazon::API::MWS::GetLowestOfferListResponseHandler;
use DE_EPAGES::Amazon::API::MWS::GetReportListResponseHandler;
use DE_EPAGES::Amazon::API::MWS::GetOrderReportResponseHandler;
use DE_EPAGES::Amazon::API::MWS::UpdateReportAcknowledgementsResponseHandler;
use DE_EPAGES::Amazon::API::Constants qw ( %AMAZON_PENDING_FEEDS );

use strict;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'MissingConstructorArgument'  => 'Need to set argument "#Argument" when calling new.',
        'InvalidMessageType'          => '"#MessageType" is not a valid msgtype. Try "orders" or "payments"',
        'InvalidRequestType'          => '"#RequestType" is not a valid request type. See RemoteAmazonMerchant docs or Amazon API',
    };
}

#ERRORDEFINITION END

sub new {
    my $class = shift;
    my %args  = @_;
    my $self  = bless {}, $class;

    # obligatory: merchant marketplace url
    for (qw/AWSURL AWSAccessKeyId AWSSecretKeyId MerchantID MarketplaceID MWSAuthToken SPAPI_EndPoint/) {
        if ($args{$_} eq '0' and $_ eq 'MerchantID') {$self->{$_} = $args{$_}} else
        {$self->{$_} = $args{$_} or Error( 'MissingConstructorArgument', { 'Argument' => $_ } )};
    }

    # optional: handler, is_logging
    for (qw/handler Logging/) {
        $self->{$_} = $args{$_} if $args{$_};
    }

    # instanciate mws adapter.
    $self->{mws} = $args{'mws'} // DE_EPAGES::Amazon::API::Object::RemoteMWS->new(
        AWSURL         => $args{AWSURL},
        AWSAccessKeyId => $args{AWSAccessKeyId},
        AWSSecretKeyId => $args{AWSSecretKeyId},
        Logging        => $self->is_logging,
        MerchantID     => $args{MerchantID},
        MarketplaceID  => $args{MarketplaceID},
        MWSAuthToken   => $args{MWSAuthToken},
        # add also endpoint and PortalConfig because needed for accessing ->doRequest
        SPAPI_EndPoint => $args{SPAPI_EndPoint},
        PortalConfig   => $args{PortalConfig},
    );

    return $self;
}

sub login {
    my $self = shift;
    my $Ret  = $self->{mws}->getOrderServiceStatus();

    return DE_EPAGES::Amazon::API::MWS::GetOrderServiceStatusResponseHandler->handle( $Ret->decoded_content() );
}

sub activateOrderScheduler {
    my $self       = shift;
    my ($Interval) = @_;
    my $hResponse = $self->{mws}->createReportSchedule(
        ReportType => $AMAZON_PENDING_FEEDS{'orders'},
        Schedule   => $Interval,
    );

    return $hResponse;
}

sub getOrderSchedulers {
    my $self       = shift;
    my $aSchedulerIDs = $self->{mws}->getReportSchedulers(
        ReportType => $AMAZON_PENDING_FEEDS{'orders'},
    );

    return $aSchedulerIDs;
}

sub deleteOrderScheduler {
    my $self       = shift;
    my ($SchedulerID) = @_;

    $self->{mws}->deleteReportScheduler($SchedulerID);
}

sub getAllPendingDocumentInfo {
    my $self = shift;
    my ($hParams, $aOrderReportIDs ) = @_;

    my $hReports = {};
    # here we get all report since the $LastReportExportDate (the last successfull run)
    unless(defined $aOrderReportIDs && (scalar @$aOrderReportIDs)) {
        $hReports = $self->{mws}->getReports($hParams);
    } else {
        my $ahReports = [];
        # get single order reports by given id's
        foreach my $ReportID (@$aOrderReportIDs){
            my $hReport = $self->{mws}->getReport($ReportID);
            push @$ahReports, $hReport;
        }
        $hReports->{'reports'} = \@$ahReports;
    }
    return $hReports;
}

sub getDocumentProcessingStatus {
    my $self = shift;
    my ($docid) = @_;

    my $Ret = $self->{mws}->getFeedSubmissionList( 'FeedSubmissionIdList.Id.1' => $docid );
    return DE_EPAGES::Amazon::API::MWS::GetFeedSubmissionListResponseHandler->handle( $Ret->decoded_content() );
}

sub getProcessingReportDocument {
    my $self = shift;
    my ($docid) = @_;

    my $Ret = $self->{mws}->getFeedSubmissionResult( 'FeedSubmissionId' => $docid );
    return DE_EPAGES::Amazon::API::MWS::GetFeedSubmissionResultResponseHandler->handle( $Ret->decoded_content() );
}

sub getLowestOfferList {
    my $self = shift;
    my ($Marketplaceid, $aASIN) = @_;

    my $i = 1;
    my %ArgsList;

    foreach my $ASIN (@$aASIN){
        $ArgsList{'ASINList.ASIN.'.$i++} = $ASIN;
    }

    $ArgsList{'MarketplaceId'} = $Marketplaceid;

    #Show only prices for items with condition "New"
    $ArgsList{'ItemCondition'} = 'New';

    #Excludes your own offer listings from the offer listings that are returned.
    $ArgsList{'ExcludeMe'} = 'True';


    my $Ret = $self->{mws}->getLowestOfferList( %ArgsList );
    return DE_EPAGES::Amazon::API::MWS::GetLowestOfferListResponseHandler->handle( $Ret->decoded_content() );
}

sub getOrderDocument {
    my $self = shift;
    my ($ReportDocID, $ReportId) = @_;

    TestString( '$ReportDocID RemoteMerchantToMWSAdapter', $ReportDocID );

    my $DocumentContent = $self->{mws}->getReportDocument($ReportDocID);

    my $Handler = DE_EPAGES::Amazon::API::MWS::GetOrderReportResponseHandler->new('identifier' => $ReportId);
    $Handler->handle( $DocumentContent );
    return $Handler;
}

sub postDocument {
    my $self = shift;
    my ( $RequestType, $Data ) = @_;
    my $Ret = $self->{mws}->submitFeed( 'FeedType' => $RequestType, 'rContent' => \$Data );
    return DE_EPAGES::Amazon::API::MWS::SubmitFeedResponseHandler->handle( $Ret->decoded_content() );
}

#============ getter ===============
sub url {
    return shift->{'AWSURL'};
}

sub marketplace {
    my $self = shift;
    my ($marketplace) = @_;
    return $self->{'MarketplaceID'} unless ( defined($marketplace) );
    $self->{'MarketplaceID'} = $marketplace;
}

sub merchant {
    my $self = shift;
    my ($merchant) = @_;
    return $self->{'MerchantID'} unless ( defined($merchant) );
    $self->{'MerchantID'} = $merchant;
}

sub mwsauthtoken {
    my $self = shift;
    my ($mwsauthtoken) = @_;
    return $self->{'MWSAuthToken'} unless ( defined($mwsauthtoken) );
    $self->{'MWSAuthToken'} = $mwsauthtoken;
}

sub is_logging {
    my $self = shift;
    my ($is_logging) = @_;
    return $self->{'Logging'} unless ( defined($is_logging) );
    $self->{'Logging'} = $is_logging;
}

sub mws {
    return shift->{'mws'};
}

sub spapi_endpoint {
    my $self = shift;
    my ($spapi_endpoint) = @_;
    return $self->{'SPAPI_EndPoint'} unless ( defined($spapi_endpoint) );
    $self->{'SPAPI_EndPoint'} = $spapi_endpoint;
}

1;
