use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Amazon::API::Constants qw (  REPORT_SCHEDULER_PERIOD );
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning BlockWarnings );
use DE_EPAGES::Core::API::Log qw ( GetLog LogError );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath LoadClassByAlias );
use DE_EPAGES::Shop::API::Feature qw( LoadFeatureByAlias );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $AmazonPlatformAlias = '';
    my $ShopAlias;
    my $Help;
    my $NoOutput;

    GetOptions(
        'help'        => \$Help,
        'nooutput'    => \$NoOutput,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'platform=s'  => \$AmazonPlatformAlias,
        'shop=s'      => \$ShopAlias,
    );

    usage() if $Help;
    usage() unless $StoreName;
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { ManageReportSchedule( ($NoOutput) ? 0 : 1, $AmazonPlatformAlias, $ShopAlias, $StoreName ) },
    );

    my $logmsg = sprintf 'need %.1f seconds', time() - $time1;
    print $logmsg. "\n" unless $NoOutput;
    GetLog()->debug("Amazon ManageReportSchedule: $logmsg");

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub ManageReportSchedule { ## no critic (ProhibitFunctionSize)
    my ( $printOutput, $AmazonPlatformAlias, $ShopAlias, $StoreName ) = @_;

    # make sure that the cartridge is installed
    unless ( ExistsCartridgeByPackage('DE_EPAGES::Amazon') ) {
        print 'Store: ' . $StoreName . ": Cartridge Amazon is not installed.\n" if ($printOutput);
        return;
    }

    my @AmazonConfigs;
    my $AmazonConfigClass = LoadClassByAlias('AmazonConfig');

    # check if feature is enabled.
    my $Feature = LoadFeatureByAlias('Amazon');

    if ( defined $ShopAlias and $ShopAlias ne 'all' ) {
        my $Shop;
        eval { $Shop = LoadObjectByPath('/Shops')->child($ShopAlias); };
        if ( ExistsError() ) {
            printf STDERR ( "[%s] WARN: Shop '%s' does not exist.\n", $StoreName, $ShopAlias );
            return;
        }
        my $aPortalConfigs = $Shop->get('PortalConfigs');
        @AmazonConfigs = grep { $_->instanceOf($AmazonConfigClass) } @{$aPortalConfigs};
    }
    else {
        @AmazonConfigs = @{ $AmazonConfigClass->objects };
    }

    foreach my $ShopPortalConfig (@AmazonConfigs) {
        my $PortalSite   = $ShopPortalConfig->get('PortalSite');
        my $Shop         = $ShopPortalConfig->getSite;
        my $FeatureValue = $Shop->featureMaxValue($Feature);
        next if ( not $FeatureValue );

        # skip if user requested a particular site.
        next if ( $AmazonPlatformAlias && $AmazonPlatformAlias ne $PortalSite->alias );

        # skip if no merchant id is set or shop is still on SOAP
        next unless defined $ShopPortalConfig->get('MWSMerchantID');

        # skip if no MWSAuthToken is set
        next unless defined $ShopPortalConfig->get('MWSAuthToken');

        my $aWarnings = BlockWarnings(
            sub {
                # check for existing scheduler and delete it
                my $aSchedulerIDs = ();
                eval { $aSchedulerIDs = $ShopPortalConfig->getOrderSchedulers(); };
                if ( ExistsError() ) {
                    my $Error = GetError();
                    $Error->addVars({ 'Path' => $ShopPortalConfig->pathString });
                    Warning($Error);
                }

                foreach my $SchedulerID (@$aSchedulerIDs) {
                    sleep 1;
                    eval { $ShopPortalConfig->deleteOrderScheduler($SchedulerID); };
                    if ( ExistsError() ) {
                        my $Error = GetError();
                        $Error->addVars({ 'Path' => $ShopPortalConfig->pathString });
                        Warning($Error);
                    }
                }

                # create new scheduler
                print 'Change ManageReportSchedule: '. $Shop->alias.' | platform: '.$PortalSite->alias."\n" if $printOutput;
                eval { $ShopPortalConfig->activateOrderScheduler(REPORT_SCHEDULER_PERIOD); };

                if ( ExistsError() ) {
                    my $Error = GetError();
                    $Error->addVars({ 'Path' => $ShopPortalConfig->pathString });
                    Warning($Error);
                }
            }
        );

        foreach my $Warning (@{$aWarnings}) {
            # ignore failed logins in scheduler calls.
            next if ($Warning->code eq 'AmazonMerchantLoginFailure' && !$printOutput);
            LogError($Warning);
            printf STDERR ( "[%s] WARN: %s: %s \n",
                $StoreName,
                $Warning->vars->{Path}//'path unknown',
                substr( $Warning->shortMessage, 0, 4096 ) ); ## no critic (ProhibitMagicNumbers)
        }
    }
}

sub usage {
    print <<END_USAGE;

    Creates a new Amazon Scheduler for pre generating order reports on the Amazon platform
    on all valid marketplaces the merchant is registered by Amazon.
    (This has nothing to do with the different Amazon platforms in the shop!)
    REPORT_SCHEDULER_PERIOD is here 15 minutes (see Amazon/API/Constants.pm)

Usage:
    perl $0 [options] [flags]

options:
    -passwd       database user password              ()
    -storename    name of store                       (Store)
    -shop         shop alias               (optional) ()
    -platform     amazon platform          (optional) (Amazon_de_DE)

flags:
    -nooutput   do not print any output
    -help       show the command line options

Example:
    perl $0 -storename Store -shop DemoShop -platform Amazon_de_DE
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
