use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath LoadClassByAlias );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );
use DE_EPAGES::XML::API::DateTimeFormatXML;


sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $AmazonPlatformAlias = '';
    my $ShopAlias;
    my $Help;
    my $NoOutput;
    my $StartDate;

    GetOptions(
        'help'        => \$Help,
        'nooutput'    => \$NoOutput,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'platform=s'  => \$AmazonPlatformAlias,
        'shop=s'      => \$ShopAlias,
        'startdate=s'  => \$StartDate,
    );

    usage() if $Help;
    usage() unless $StoreName;
    usage() unless $StartDate;


    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub {
            SetLastReportExportDate( {
                    'PrintOutput' => ($NoOutput) ? 0 : 1,
                    'AmazonPlatformAlias' => $AmazonPlatformAlias,
                    'ShopAlias'           => $ShopAlias,
                    'StartDate'           => $StartDate,
                    'StoreName'           => $StoreName,
                }
            );
        },
    );
    my $time2   = time;
    my $seconds = $time2 - $time1;
    my $logmsg  = sprintf( 'need %.3f seconds', $seconds );
    print $logmsg. "\n" unless $NoOutput;
    GetLog()->debug("Set Attribute LastReportExportDate: $logmsg");

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub SetLastReportExportDate {    ## no critic (ProhibitFunctionSize)
    my ($hInput) = @_;

    # make sure that the cartridge is installed
    unless ( ExistsCartridgeByPackage('DE_EPAGES::Amazon') ) {
        print 'Store: ' . $hInput->{'StoreName'} . ": Cartridge Amazon is not installed.\n" if ( $hInput->{'PrintOutput'} );
        return;
    }

    my @AmazonConfigs     = ();
    my $AmazonConfigClass = LoadClassByAlias('AmazonConfig');
    if ( defined $hInput->{'ShopAlias'} and $hInput->{'ShopAlias'} ne 'all' ) {
        my $Shop;
        eval { $Shop = LoadObjectByPath('/Shops')->child( $hInput->{'ShopAlias'} ); };
        if ( ExistsError() ) {
            printf STDERR ( "[%s] WARN: Shop '%s' does not exist.\n", $hInput->{'StoreName'}, $hInput->{'ShopAlias'} );
            return;
        }

        my $aPortalConfigs = $Shop->get('PortalConfigs');
        @AmazonConfigs = grep { $_->instanceOf($AmazonConfigClass) } @{$aPortalConfigs};
    }
    else {
        @AmazonConfigs = @{ $AmazonConfigClass->objects };
    }

    my $TIME_PARSER = DE_EPAGES::XML::API::DateTimeFormatXML->new;
    my $LastReportExportDate = $TIME_PARSER->parse_datetime($hInput->{'StartDate'});

    print 'Store: ' . $hInput->{'StoreName'} . " :Found: ".scalar(@AmazonConfigs)." AmazonConfigs.\n"
        if ( $hInput->{'PrintOutput'} );

    foreach my $AConfig (@AmazonConfigs) {
        my $PortalSite   = $AConfig->get('PortalSite');
        # skip if user requested a particular site.
        if ( $hInput->{'AmazonPlatformAlias'} && $hInput->{'AmazonPlatformAlias'} ne $PortalSite->alias ){
            print 'Store: ' . $hInput->{'StoreName'} . " :AmazonPlatformAlias not found: ".$hInput->{'AmazonPlatformAlias'}.".\n";
            next;
        }
        # skip if no SPAPI_AccessToken is set
        unless (defined $AConfig->get('SPAPI_AccessToken')){
            print 'Store: ' . $hInput->{'StoreName'} . " :Ignore AmazonConfig: ".$AConfig->id." because SPAPI_AccessToken not set for: ".$hInput->{'AmazonPlatformAlias'}.".\n"
                if ( $hInput->{'PrintOutput'} );
            next;
        }
        # now we can set the attribute
        print 'Store: ' . $hInput->{'StoreName'} . " :Set LastReportExportDate for AmazonConfig: ".$AConfig->id.".\n"
            if ( $hInput->{'PrintOutput'} );
        $AConfig->set({'LastReportExportDate' => $LastReportExportDate});
    }
}

sub usage {
    print <<END_USAGE;

Sets the attribute LastReportExportDate to the given date for shop/shops platform/platforms.
AmazonConfigs without a SPAPI_AccessToken will be ignored.

(Attribute LastReportExportDate is used for having a start date for the next scheduler run to get reports from Amazon.)

Usage:
    perl $0 [options] [flags]

options:
    -passwd       database user password              ()
    -storename    name of store                       (Store)
    -shop         shop alias             (optional)   (all)
    -platform     amazon platform        (optional)   (Amazon_de_DE)
    -startdate    start date to set (for fetching order documents later by scheduler)
flags:
    -nooutput   do not print any output
    -help       show the command line options

Example:
    perl $0 -storename Store -startdate 2022-07-13T07:46:49
    perl $0 -storename Store -shop DemoShop -platform Amazon_de_DE -startdate 2022-07-13T07:46:49
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
