#========================================================================================
# package      DE_EPAGES::Spreedly::UI::Shop
# base         DE_EPAGES::Order::UI::Shop
# state        public
#----------------------------------------------------------------------------------------
# description  UI Controller for shop
#========================================================================================
package DE_EPAGES::Spreedly::UI::Shop;
use base DE_EPAGES::Order::UI::Shop;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( AddDuration isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Presentation::API::DereferrerToken qw ( ValidateDereferrerToken );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObjectByGUID );
use DE_EPAGES::Search::API::Object::PagerSearch qw ( ReplaceWildcards ReplaceWildcardsWithoutPosOne );
use DE_EPAGES::Search::API::SearchDefinition qw ( GetSearchDefinition );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Spreedly::API::Constants qw (
    INTERNAL_CARTRIDGE_NAME
    THREE_DS_AVAILABLE_GATEWAYS
    PAYMENT_STATUS_FAILED );

use strict;


#========================================================================================
# function     TransactionPaymentName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Name = $Handler->TransactionPaymentName($Servlet);
#----------------------------------------------------------------------------------------
# description  Name of PaymentType for search/view transactions.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
# return       $Name | name of payment type | string
#========================================================================================
sub TransactionPaymentName {
    my $self = shift;
    my ($Servlet) = @_;

    if (defined $Servlet) {
        my $Params = $Servlet->params();

        my $PaymentName = $Params->{'ViewAction'};
        $PaymentName =~ s/.*Search(\w+)Transactions/$1/g;

        return $PaymentName;
    }
    return INTERNAL_CARTRIDGE_NAME;
}


#========================================================================================
# function     SpreedlyPaymentCallback
# state        public
#----------------------------------------------------------------------------------------
# syntax       SpreedlyPaymentCallback( $Servlet );
#----------------------------------------------------------------------------------------
# description  When using Spreedly 3DS2 a Fallback to 3DS1 can occur
#               This means we have to support callback updates to the order
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub SpreedlyPaymentCallback {
    my $self = shift;
    my ($Servlet) = @_;

    eval {
        $self->_authenticateCallback($Servlet);
        $self->_handleSpreedlyCallback($Servlet);
    };
    if ( ExistsError() ) {
        $self->_throwWarning(GetError()->message, $Servlet);
    }
    return;
}

#========================================================================================
# function     SearchTransactions
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SearchTransactions( $Servlet );
#----------------------------------------------------------------------------------------
# description  Searches for transactions of the current payment type.
#               The TLE variable #Pager contains the search results.
#               See also <function TransactionPaymentName>.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $hVars | TLE variables for 'Pager' | ref.hash
#========================================================================================
sub SearchTransactions {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    my $SearchActionName = 'Search' . $self->TransactionPaymentName($Servlet) . 'Transactions';
    my $SearchForm = $Servlet->form->form($Shop, $SearchActionName);

    my $SearchTemplateName = $self->TransactionPaymentName($Servlet) . 'Transactions';
    $SearchForm->{'Search'} = $SearchTemplateName;

    #--- default values
    if (!defined $Servlet->params('OrderBy')) {
        $Servlet->params('OrderBy', 'TransTime');
        $Servlet->params('OrderDesc', 1);
    }

    $Servlet->vars('ViewSearchAction', 'MBO-' . $SearchActionName);

    return $self->_ExecuteTransactionSearch($Servlet, $SearchForm);
}

#========================================================================================
# function     _ExecuteTransactionSearch
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->_ExecuteTransactionSearch( $Servlet, $SearchForm);
#----------------------------------------------------------------------------------------
# description  Search and Pager for functions <function ViewTransactions> and <function SearchTransactions>.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $SearchForm | form values | ref.hash.string
# return       $hPagerVars | pager variables that can be used inside a template (cf. $Pager-><function page>) | ref.hash
#========================================================================================
#========================================================================================
# function     ExecuteTransactionSearch
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->_ExecuteTransactionSearch( $Servlet, $SearchForm);
#----------------------------------------------------------------------------------------
# description  Search and Pager for functions <function ViewTransactions> and <function SearchTransactions>.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $SearchForm | form values | ref.hash.string
# return       $hVars | TLE variables for 'Pager' | ref.hash
#========================================================================================
sub _ExecuteTransactionSearch {
    my $self = shift;
    my ($Servlet, $SearchForm) = @_;

    delete $SearchForm->{$_} foreach grep { not defined $SearchForm->{$_} } keys %$SearchForm;
    $SearchForm->{'Search'} = 'Transactions' unless exists $SearchForm->{'Search'};
    my $Shop = $Servlet->object;
    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('LineItemPayment' . $self->TransactionPaymentName()), $SearchForm->{'Search'});

    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $Shop,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'    => $Shop->id,
            'LanguageID' => $Servlet->languageID,
            'IsVisible'  => 1,
        },
        'UriParameters'  => {
            'Search' => $SearchForm->{'Search'},
        },
    );

    my $hSearchOptions = $self->SearchOptions();
    $Options{'Parameters'}{$_} = $hSearchOptions->{$_} foreach keys %$hSearchOptions;

    delete $SearchForm->{'Search'};
    foreach my $Parameter (keys %$SearchForm) {
        if ( $Parameter eq 'ProductAliasString' ) {

            ## UNITY-4143
            #replace '*' with '%' and add % to the end for (Alias/Productnumber)
            $Options{'Parameters'}{$Parameter} = ReplaceWildcardsWithoutPosOne($SearchForm->{$Parameter});
        } elsif ($Parameter =~ /String/) {
            # replace * with %
            $Options{'Parameters'}{$Parameter} = ReplaceWildcards($SearchForm->{$Parameter});

        } elsif ($Parameter =~ /(.*)On$/) {
            # status = 0/1/2 egal/yes/no
            my $Status = $1;
            if ($SearchForm->{$Parameter} == 1) {
                $Options{'Parameters'}{'Is'.$Status} = 1;
            } elsif ($SearchForm->{$Parameter} == 2) {
                $Options{'Parameters'}{'Not'.$Status} = 1;
            }
        } elsif ($Parameter eq 'TransTimeTo'){
            #add 1 day to the searchdate
            $Options{'Parameters'}{$Parameter} = AddDuration($SearchForm->{$Parameter}->clone(), 'days', 1);

        }else{
            $Options{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        }

        # EPG-12969: use original input values in pager URL
        if( $Parameter =~ /^TransTime(From|To)$/ ) {
            $Options{'UriParameters'}{$Parameter} = $Servlet->form->value($Parameter);
        }
        else {
            $Options{'UriParameters'}{$Parameter} = $SearchForm->{$Parameter};
        }
    }

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    return $Search->page;
}

sub _handleSpreedlyCallback {
    my $self = shift;
    my ($Servlet) = @_;

    my $LineItemPaymentSpreedly = $self->_getLineItemPaymentSpreedly($Servlet->params('PaymentLineItemGUID'));
    my $TransactionToken = $LineItemPaymentSpreedly->getTransactionToken();

    my $LineItemContainer = $LineItemPaymentSpreedly->parent();

    if(not isValid($LineItemContainer) or not $LineItemContainer->instanceOf('LineItemContainer')) {
        Error('Invalid Order Container');
    }

    my $PaymentMethod = $LineItemPaymentSpreedly->get('PaymentMethod');
    my $PaymentType = $PaymentMethod->get('PaymentType');
    my $GatewayName = $PaymentType->get('GatewayName');

    if (not ($GatewayName ~~ THREE_DS_AVAILABLE_GATEWAYS)) {
        Error('Invalid Gateway');
    }

    my $BasketOrder = $LineItemContainer->parent();

    if(not isValid($BasketOrder)) {
        Error('Basket undefined');
    }
    if(not $BasketOrder->instanceOf('Basket')) {
        $self->_LogPayment($PaymentType->alias(), 'Object is not a basket', {'class' => LoadClassByAlias($BasketOrder->alias())});
    }
    my $LineItemPaymentStatus = $LineItemPaymentSpreedly->getTransactionStatus();

    if (not isValid($LineItemPaymentStatus) or
        ((not $LineItemPaymentSpreedly->isPaymentSuccessful()) && $LineItemPaymentStatus ne PAYMENT_STATUS_FAILED) ) {
        my $hTransaction = $self->_requestSpreedlyTransactionByToken($BasketOrder, $GatewayName, $TransactionToken);
        my $TransactionPaymentStatus = $LineItemPaymentSpreedly->getPaymentStatusFromTransactionState($hTransaction->{'state'});
        $self->_LogPayment($PaymentType->alias(), 'updateTransactionStatus', {'Transaction' => $hTransaction});
        if ($TransactionPaymentStatus ne $LineItemPaymentStatus) {
            $LineItemPaymentSpreedly->updateTransactionStatus($TransactionPaymentStatus, $hTransaction);
        }
    }

    return;
}

sub _getLineItemPaymentSpreedly {
    my $self = shift;
    my ($GUID) = @_;

    my $LineItemPaymentSpreedly;

    Error('Undefined GUID') unless isValid($GUID);

    eval {
        $LineItemPaymentSpreedly = $self->_LoadObjectByGUID($GUID);
    };
    if (ExistsError()) {
        Error('GUID does not exist in Database');
    }

    if(not isValid($LineItemPaymentSpreedly) or not $LineItemPaymentSpreedly->instanceOf('LineItemPaymentSpreedly')) {
        Error('Invalid GUID');
    }

    return $LineItemPaymentSpreedly;
}

sub _authenticateCallback {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $ObjectId = $Servlet->params('ObjectID');
    my $PaymentLineItemGuid = $Servlet->params('PaymentLineItemGUID');

    my $PayLoad = sprintf('%s&%s&%s', $Shop->alias(), $ObjectId, $PaymentLineItemGuid);
    my $Token = $Servlet->params('DereferrerToken');

    $self->_ValidateDereferrerToken($Token, $PayLoad);

    return;
}

sub _LogPayment {
    my $self = shift;
    my ($PaymentTypeAlias, $Name, $DebugStuff) = @_;

    LogPayment($PaymentTypeAlias, $Name, $DebugStuff);

    return;
}

sub _ValidateDereferrerToken {
    my $self = shift;
    my ($Token, $URL) = @_;
    ValidateDereferrerToken($Token, $URL);
}

sub _LoadObjectByGUID {
    my $self = shift;
    my ($GUID) = @_;
    return LoadObjectByGUID($GUID);
}

sub _requestSpreedlyTransactionByToken {
    my $self = shift;
    my ($BasketOrOrder, $GatewayName, $TransactionToken) = @_;
    my $hTransaction;

    eval {
        $hTransaction = $self->_showTransaction($BasketOrOrder, $GatewayName, $TransactionToken);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        $self->_LogPayment('Error on _requestSpreedlyTransactionByToken', {'Error' => $Error->message});
        Error('Error connecting to Spreedly');
    }
    if ( $hTransaction->{'successful'}) {
        return $hTransaction;
    }
    Error('Error returned from Spreedly');
}

sub _showTransaction {
    my $self = shift;
    my ($BasketOrder, $GatewayName, $TransactionToken) = @_;

    my $Shop = $BasketOrder->getSite();
    my $PurchaseController = DE_EPAGES::Spreedly::API::PurchaseController->new();
    my $hTransaction = $PurchaseController->ShowTransaction($Shop, $GatewayName, $TransactionToken);

    return $hTransaction;
}

sub _throwWarning {
    my $self = shift;
    my ($Message, $Servlet) = @_;

    my $Response = $Servlet->response();

    $Response->code(406);
    Warning( 'Spreedly_SpreedlyPaymentCallback' , {'Message' => $Message });

    return;
}

1;